﻿#include "jsonarray.h"
#include <QMetaObject>
#include <QJsonDocument>
#include <QJsonArray>
#include <QJsonObject>
#include "jsonobj.h"
#include <QDebug>
#include <QQmlEngine>

JSONArray::JSONArray(QObject* parent) : QObject(parent)
{
    this->setObjectName("JSONArray");
    m_jsonVec.clear();
    qmlEngine(this)->setObjectOwnership(this, QQmlEngine::CppOwnership);
}

JSONArray::~JSONArray()
{
    m_jsonVec.clear();
}

JSONArray::JSONArray(const JSONArray &jsonArr)
{
    this->setObjectName("JSONArray");
    this->m_jsonVec = jsonArr.m_jsonVec;
}

QString JSONArray::toJSONString()
{
    QJsonArray jsonArr;
    for (int i = 0; i !=  m_jsonVec.size(); ++i)
    {
        QVariant itemVar = m_jsonVec.at(i);
        QJsonValue jsonitem = getSubItem(itemVar);
        jsonArr.push_back(jsonitem);
    }

    QJsonDocument jsonDoc(jsonArr);
    return QString(jsonDoc.toJson());
}

void JSONArray::add(QVariant elem)
{
    m_jsonVec.push_back(elem);
}
void JSONArray::add(QObject* elem)
{
    m_jsonVec.push_back(QVariant::fromValue(elem));
}
void JSONArray::add(int index, QVariant elem)
{
    m_jsonVec.insert(index, elem);
}

void JSONArray::addAll(QVariant collection)
{
    QObject *objCollection = collection.value<QObject*>();
    if (!objCollection)
        return;
    int size = 0;
    QMetaObject::invokeMethod(objCollection,
                              "size",
                              Qt::DirectConnection,
                              Q_RETURN_ARG(int, size));
    for (int i = 0; i != size; i++)
    {
        QVariant item;
        QMetaObject::invokeMethod(objCollection,
                                  "get",
                                  Qt::DirectConnection,
                                  Q_RETURN_ARG(QVariant, item),
                                  Q_ARG(int, i));
        m_jsonVec.push_back(item);
    }
}

void JSONArray::addAll(int index, QVariant collection)
{
    int arraySize = m_jsonVec.size();
    int insertPos = arraySize > index ? index : arraySize;

    QObject *objCollection = collection.value<QObject*>();
    int size = 0;
    QMetaObject::invokeMethod(objCollection,
                              "size",
                              Qt::DirectConnection,
                              Q_RETURN_ARG(int, size));
    for (int i = 0; i != size; i++)
    {
        QVariant item;
        QMetaObject::invokeMethod(objCollection,
                                  "get",
                                  Qt::DirectConnection,
                                  Q_RETURN_ARG(QVariant, item),
                                  Q_ARG(int, i));
        m_jsonVec.insert(insertPos + i, item);
    }
}

void JSONArray::clear()
{
    m_jsonVec.clear();
}

bool JSONArray::contains(QVariant elem)
{
    return m_jsonVec.contains(elem);
}

bool JSONArray::containsAll(QVariant collection)
{
    QObject *objCollection = collection.value<QObject*>();
    if (!objCollection)
        return false;
    int size = 0;
    QMetaObject::invokeMethod(objCollection,
                              "size",
                              Qt::DirectConnection,
                              Q_RETURN_ARG(int, size));
    for (int i = 0; i != size; i++)
    {
        QVariant item;
        QMetaObject::invokeMethod(objCollection,
                                  "get",
                                  Qt::DirectConnection,
                                  Q_RETURN_ARG(QVariant, item),
                                  Q_ARG(int, i));

        if (!m_jsonVec.contains(item))
            return false;
    }

    return true;
}

QVariant JSONArray::get(int index)
{
//    qDebug()<<"cpp value "<<m_jsonVec.at(index);
    return m_jsonVec.at(index);
}

int JSONArray::indexOf(QVariant elem)
{
    return m_jsonVec.indexOf(elem);
}

bool JSONArray::isEmpty()
{
    return m_jsonVec.isEmpty();
}

QVariant JSONArray::remove(int index)
{
    if (index < m_jsonVec.count())
    {
        QVariant tmpVal = m_jsonVec.takeAt(index);
        return tmpVal;
    }
	return QVariant();
}

bool JSONArray::remove(QVariant elem)
{
    int index = indexOf(elem);
    if (index != -1)
    {
        m_jsonVec.removeAt(index);
        return true;
    }
    else
    {
        return false;
    }
}

bool JSONArray::removeAll(QVariant collection)
{
    bool isRemove = false;
    QVector<int> removeList;

    {
        QObject *objCollection = collection.value<QObject*>();
        if (!objCollection)
            return false;
        int size = 0;
        QMetaObject::invokeMethod(objCollection,
                                  "size",
                                  Qt::DirectConnection,
                                  Q_RETURN_ARG(int, size));
        for (int i = 0; i != size; i++)
        {
            QVariant item;
            QMetaObject::invokeMethod(objCollection,
                                      "get",
                                      Qt::DirectConnection,
                                      Q_RETURN_ARG(QVariant, item),
                                      Q_ARG(int, i));

            for (int j = 0; j != m_jsonVec.count(); ++j)
            {
                if (m_jsonVec.at(j) == item)
                {
                    removeList.push_back(i);
                }
            }
        }
    }

    for (int k = removeList.count() - 1; k >= 0; --k)
    {
        m_jsonVec.removeAt(removeList.at(k));
    }

    if (removeList.count() > 0)
        isRemove = true;
    return isRemove;
}

int JSONArray::size() const
{
    return m_jsonVec.size();
}

QJsonValue JSONArray::getSubItem(QVariant item)
{
    // JSONObj类型
    if (item.canConvert<JSONObj*>())
    {
        JSONObj *jsonObj = item.value<JSONObj*>();
        QMap<QString, QVariant> tmpJsonMap = jsonObj->m_jsonMap;
        QJsonObject tmpJsonObj;
        for (auto it = tmpJsonMap.begin(); it != tmpJsonMap.end(); it++)
        {
            QJsonValue itemJsonObj = getSubItem(it.value());
            tmpJsonObj.insert(it.key(), itemJsonObj);
        }
        return tmpJsonObj;
    }
    // JSONArray类型
    else if (item.canConvert<JSONArray*>())
    {
        JSONArray *jsonArr = item.value<JSONArray*>();
        QVector<QVariant> tmpJsonVec = jsonArr->m_jsonVec;
        QJsonArray tmpJsonArr;
        for (auto it = tmpJsonVec.begin(); it != tmpJsonVec.end(); it++)
        {
            QJsonValue itemJsonObj = getSubItem(*it);
            tmpJsonArr.push_back(itemJsonObj);
        }
        return tmpJsonArr;
    }
    // Vector类型
//    else if (item.canConvert<Vector*>())
//    {
//        Vector *tmpVecObj = item.value<Vector*>();
//        QVector<QVariant> tmpJsonVec = tmpVecObj->getInlineVec();
//        QJsonArray tmpJsonArr;
//        for (auto it = tmpJsonVec.begin(); it != tmpJsonVec.end(); it++)
//        {
//            QJsonValue itemJsonObj = getSubItem(*it);
//            tmpJsonArr.push_back(itemJsonObj);
//        }
//        return tmpJsonArr;
//    }
    else
    {
        QJsonValue retJsonObj = QJsonValue::fromVariant(item);
        return retJsonObj;
    }
}

void JSONArray::set(int index, QVariant elem)
{
    m_jsonVec.replace(index, elem);
}

QVariantList JSONArray::toArray()
{
    return m_jsonVec.toList();
}

