﻿#include "jsonobj.h"
#include <QMetaObject>
#include <QJsonDocument>
#include <QJsonArray>
#include <QJsonObject>
#include <QQmlEngine>
#include <QDebug>
#include "jsonarray.h"

JSONObj::JSONObj(QObject* parent) : QObject(parent)
{
    this->setObjectName("JSONObj");
    m_jsonMap.clear();
}

JSONObj::JSONObj(const JSONObj &jsonObj)
{
    this->m_jsonMap = jsonObj.m_jsonMap;
    this->setObjectName("JSONObj");
    qmlEngine(this)->setObjectOwnership(this, QQmlEngine::CppOwnership);
}

//JSONObj::JSONObj(QJSValue map)
//{
//    this->setObjectName("JSONObj");
//    QObject *mapObj = map.toQObject();
//    JsMap *realMap = qobject_cast<JsMap*>(mapObj);
//    QList<QVariant> keys = realMap->keySet();
//    for (int i = 0; i < keys.size(); ++i)
//    {
//        QVariant key = keys.at(i);
//        QVariant value = realMap->get(key);
//        m_jsonMap.insert(key.toString(), value);
//    }
//}

//JSONObj::JSONObj(Vector *vector)
//{
//    this->setObjectName("JSONObj");
//    QVariant value = QVariant::fromValue(vector);
//    m_jsonMap.insert("queryresult", value);
//}

JSONObj::~JSONObj()
{
    m_jsonMap.clear();
}

QString JSONObj::toJSONString()
{
    QJsonObject jsonObj;
    for (auto it = m_jsonMap.begin(); it != m_jsonMap.end(); it++)
    {
        QJsonValue itemObj = getSubItem(it.value());
        qDebug() << it.key() << itemObj.isString() << itemObj.toString();
        jsonObj.insert(it.key(), itemObj);
    }

    QJsonDocument jsonDoc(jsonObj);
    return QString(jsonDoc.toJson());
}

QVariant JSONObj::getJSONObject(QString key)
{
    return m_jsonMap.value(key);
}

QVariant JSONObj::getJSONArray(QString key)
{
    return m_jsonMap.value(key);
}

int JSONObj::size()
{
    return m_jsonMap.size();
}

bool JSONObj::isEmpty()
{
    return m_jsonMap.isEmpty();
}

void JSONObj::put(QString key, QVariant value)
{
    m_jsonMap.insert(key, value);
}

QVariant JSONObj::get(QString key)
{
    return m_jsonMap.value(key);
}

void JSONObj::remove(QString key)
{
    m_jsonMap.remove(key);
}

void JSONObj::clear()
{
    m_jsonMap.clear();
}

QVariantList JSONObj::keySet()
{
    QStringList keys = m_jsonMap.keys();
    QVariantList keySet;
    for (int i = 0; i != keys.size(); ++i)
    {
        QVariant tmpKey(keys.at(i));
        keySet.push_back(tmpKey);
    }
    return keySet;
}

QVariantList JSONObj::values()
{
    return m_jsonMap.values();
}

JSONObj *JSONObj::entrySet()
{
    return this;
}

JSONObj *JSONObj::iterator()
{
    m_iterator = new QMapIterator<QString, QVariant>(m_jsonMap);
    return this;
}

QString JSONObj::getString(QString elem)
{
    return m_jsonMap.value(elem).toString();
}

QString JSONObj::toString()
{
    return m_jsonMap.value("value").toString();
}

QVariant JSONObj::getData()
{
    return m_jsonMap.value("value");
}

double JSONObj::getDouble(QString elem)
{
    QVariant douVal = m_jsonMap.value(elem);
    return douVal.toDouble();
}

JSONObj &JSONObj::operator =(const JSONObj &obj)
{
    m_jsonMap = obj.m_jsonMap;
    return *this;
}

bool JSONObj::hasNext()
{
    if (m_iterator)
    {
        return m_iterator->hasNext();
    }

    return false;
}

JSONObj *JSONObj::next()
{
    if (m_iterator)
    {
        m_iterator->next();
        return this;
    }

    return nullptr;
}

QString JSONObj::getKey()
{
    return m_iterator->key();
}

QVariant JSONObj::getValue()
{
    return m_iterator->value();
}

QJsonValue JSONObj::getSubItem(QVariant item)
{
    // JSONObj类型
    if (item.canConvert<JSONObj*>())
    {
        JSONObj *jsonObj = item.value<JSONObj*>();
        QMap<QString, QVariant> tmpJsonMap = jsonObj->m_jsonMap;
        QJsonObject tmpJsonObj;
        for (auto it = tmpJsonMap.begin(); it != tmpJsonMap.end(); it++)
        {
            QJsonValue itemJsonObj = getSubItem(it.value());
            tmpJsonObj.insert(it.key(), itemJsonObj);
        }
        return tmpJsonObj;
    }
    // JSONArray类型
    else if (item.canConvert<JSONArray*>())
    {
        JSONArray *jsonArr = item.value<JSONArray*>();
        QVector<QVariant> tmpJsonVec = jsonArr->m_jsonVec;
        QJsonArray tmpJsonArr;
        for (auto it = tmpJsonVec.begin(); it != tmpJsonVec.end(); it++)
        {
            QJsonValue itemJsonObj = getSubItem(*it);
            tmpJsonArr.push_back(itemJsonObj);
        }
        return tmpJsonArr;
    }
//    // Vector类型
//    else if (item.canConvert<Vector*>())
//    {
//        Vector *tmpVecObj = item.value<Vector*>();
//        QVector<QVariant> tmpJsonVec = tmpVecObj->getInlineVec();
//        QJsonArray tmpJsonArr;
//        for (auto it = tmpJsonVec.begin(); it != tmpJsonVec.end(); it++)
//        {
//            QJsonValue itemJsonObj = getSubItem(*it);
//            tmpJsonArr.push_back(itemJsonObj);
//        }
//        return tmpJsonArr;
//    }
    else
    {
        QJsonValue retJsonObj = QJsonValue::fromVariant(item);
        return retJsonObj;
    }
}

